
import React, { Component, useEffect, useState } from "react";
import { View, TouchableOpacity, Image,Text,FlatList,NativeEventEmitter} from 'react-native';
import Logger from "./lib/Logger";
import JitsiMeetJS from 'lib-jitsi-meet/lib-jitsi-meet.min';
import { isScreenSharingTrack, useIsMouseMoving } from "./lib/utils";
import MainVideoContainerComponent from './components/MainVideoComponent'
import {RTCView} from 'react-native-webrtc';

import AudioButton from './components/AudioButton';
import AudioModeButton from './components/AudioModeButton';
import VideoButton from './components/VideoButton';
import SwitchCameraButton from './components/SwitchCameraButton';
import HangUpButton from './components/HangUpButton';
import DoubleTap from './components/DoubleTap';
import TextAvatar from './components/TextAvatar';
import AudioMode from './AudioMode'

let roomName = "Conference";
let connection = null;
let room = null;
let isJoined = false;
const MAX_VIDEO_STREAM_LIMIT = -1;

const channelLastN = MAX_VIDEO_STREAM_LIMIT;

const SCREEN_SHARING_STARTED = "SCREEN_SHARING_STARTED"; 
const SCREEN_SHARING_STOPPED = "SCREEN_SHARING_STOPPED";

const getTotalVideoStreams = (users) => {
  let count = 0;
  users.forEach((user) => {
    if (!user.isVideoMuted) {
      count++;
    }
  });
  return count;
};

const initOptions = {
  disableAudioLevels: false,
  desktopSharingChromeExtId: "mbocklcggfhnbahlnepmldehdhpjfcjp",
  desktopSharingChromeDisabled: false,
  desktopSharingChromeSources: ["screen", "window"],
  desktopSharingChromeMinExtVersion: "0.1",
  desktopSharingFirefoxDisabled: false,
  enableTalkWhileMuted: true,
  enableNoisyMicDetection: true,
  enableLayerSuspension: true,
  channelLastN,
};

const options = {
  hosts: {
    domain: "rtc-in.cometchat.io",
    muc: "conference.rtc-in.cometchat.io",
  },
  enableNoisyMicDetection: true,
  bosh: "https://rtc-in.cometchat.io/http-bind",
  clientNode: "http://jitsi.org/jitsimeet",
  channelLastN,
  enableLayerSuspension: true,
  enableTalkWhileMuted: true,
};

const confOptions = {
  openBridgeChannel: true,
  enableNoisyMicDetection: true,
  channelLastN,
  enableLayerSuspension: true,
  enableTalkWhileMuted: true,
};

var eventEmitter = new NativeEventEmitter(AudioMode);

let TAG = "CallingComponent";
export default class CallingComponent extends Component {
  constructor(props) {
    super(props);
    this.participantsWhoPinnedMe = new Set();

    const localUser = {
      name: this.props.username || "Unnamed",
      status: "active",
      isVideoMuted: false,
      isAudioMuted: false,
    };

    this.state = {
      mySessionId: this.props.sessionID,
      myUserName: this.props.username,
      status: "active",
      myAvatar: this.props.avatar,
      remoteTracks: [],
      users: [localUser],
      dominantSpeakerId: null,
      isScreenShareOn: false,
      isGridMode: false,
      gridItemHeight: 100,
      gridItemWidth: 100,
      pinnedUserId: null,
      hideAudioMuteButton : false,
      hidePauseVideoButton : false,
      hideEndCallButton : false,
      hideEndCallButton : false,
      hideVideoSwitchButton : false,
      hideAudioModeButton : false,
      showUserPopUP : false,
      isVideoMirrored : true,
    };
  }

  componentDidMount() {
    this.createLocalTracks();
    this.init(this.state.mySessionId);
    JitsiMeetJS.setLogLevel(JitsiMeetJS.logLevels.ERROR);
    Logger.log(TAG,"Component did mount ",'');
    this.onDoubleTap = this.onDoubleTap.bind(this);
    this.renderUserItem = this.renderUserItem.bind(this);
    this.onAudioMute = this.onAudioMute.bind(this);
    this.onVideoMute = this.onVideoMute.bind(this);
    this.onHangUp = this.onHangUp.bind(this);

    this.addNativeEventListner();
    this.initializeRemovingInterruptedUsers();

  }

  componentDidUpdate(_props, prevState) {
    if (this.state.users !== prevState.users) {
        this.initializeRemovingInterruptedUsers();
    }
  }

  initializeRemovingInterruptedUsers() {
    this.removeInterruptedUsersTimeoutId = setTimeout(() => {
      const filteredUsers = this.state.users.filter(
        (user) => user.status !== "interrupted"
      );
      this.setState({
        users: filteredUsers,
      });
      clearTimeout(this.removeInterruptedUsersTimeoutId);
    }, 30000);
  }

  get localUser() {
    return this.state.users[0];
  }
  dominantSpeakers = [];

  unload = (e) => {
    const { localUser } = this;
    if (localUser.audioTrack) {
      localUser.audioTrack.dispose();
    }
    if (localUser.videoTrack) {
      localUser.videoTrack.dispose();
    }

    if (room) room.leave();
    connection.disconnect();
  };

  init = (sessionID) => {
    roomName = sessionID;
    JitsiMeetJS.init(initOptions);
    JitsiMeetJS.setLogLevel(JitsiMeetJS.logLevels.ERROR);
    this.makeConnection();
  };

  makeConnection() {
    connection = new JitsiMeetJS.JitsiConnection(null, null, options);
    connection.connect();
    this.addEventListener();
  }


  addNativeEventListner(){
    //this.eventListener = new NativeEventEmitter(AudioMode);
    eventEmitter.addListener('CallEvent',(type) =>{
        Logger.log("Native received : ",type)
        if(type.type == "NativeEndCall"){
           this.onHangUp()
        }
    });
  }

  addEventListener() {
    connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
      this.onConnectionSuccess
    );
    connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_FAILED,
      this.onConnectionFailed
    );
    // connection.addEventListener(
    //   JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
    //   this.onDisconnect
    // );
  }

  onConnectionSuccess = () => {
    room = connection.initJitsiConference(roomName, confOptions);
    window.room = room;
    Logger.log(TAG, "onConnectionSuccess Called......", room);
    const myUserID = room.myUserId() || "";
    const users = [...this.state.users];
    const localUser = users[0];
    users[0] = {
      ...localUser,
      id: myUserID,
    };
    this.setState({
      users,
      dominantSpeakerId: room.lastDominantSpeaker || myUserID,
    });
    room.on(JitsiMeetJS.events.conference.TRACK_ADDED, this.onRemoteTrack);

    room.on(
      JitsiMeetJS.events.conference.TRACK_REMOVED,
      this.onRemoteTrackRemoved
    );
    room.on(
      JitsiMeetJS.events.conference.CONFERENCE_JOINED,
      this.onConferenceJoined
    );
    room.on(JitsiMeetJS.events.conference.USER_JOINED, this.onUserJoined);
    room.on(JitsiMeetJS.events.conference.USER_LEFT, this.onUserLeft);
    room.on(
      JitsiMeetJS.events.conference.TRACK_MUTE_CHANGED,
      this.onTrackMuted
    );
    room.on(
      JitsiMeetJS.events.conference.DOMINANT_SPEAKER_CHANGED,
      this.onDominantSpeakerChanged
    );
    room.on(
      JitsiMeetJS.events.connectionQuality.REMOTE_STATS_UPDATED,
      this.onRemoteStatsUpdated
    );
    room.on(
      JitsiMeetJS.events.connectionQuality.LOCAL_STATS_UPDATED,
      this.onLocalStatsUpdated
    );

    room.on(
      JitsiMeetJS.events.conference.PARTICIPANT_CONN_STATUS_CHANGED,
      (id, status) => {
        const users = this.state.users.map((user) => {
          if (user.id === id) {
            return {
              ...user,
              status,
            };
          }

          return user;
        });

        this.setState({ users });
      }
    );

    room.addCommandListener(SCREEN_SHARING_STARTED, ({ value }) => {
      this.setState({ pinnedUserId: value });
      if(room){
        room.pinParticipant(value);
      }
    });

    room.addCommandListener(SCREEN_SHARING_STOPPED, ({ value }) => {
      if (value === this.state.pinnedUserId) {
        room.pinParticipant(null);
        this.setState({
          pinnedUserId: null,
        });
      }
    });

    // room.on(
    //   JitsiMeetJS.events.conference.TRACK_AUDIO_LEVEL_CHANGED,
    //   (userID, audioLevel) => Logger.log(`${userID} - ${audioLevel}`)
    // );
    // room.on(JitsiMeetJS.events.conference.PHONE_NUMBER_CHANGED, () =>Logger.log(`${room.getPhoneNumber()} - ${room.getPhonePin()}`));
    // room.on(JitsiMeetJS.events.conference.DISPLAY_NAME_CHANGED,(userID, displayName) => console.log(`${userID} - ${displayName}`));
    room.setDisplayName(this.state.myUserName);
    room.setReceiverVideoConstraint(180);
    room.join();
  };

  onLocalStatsUpdated(stats) {
    //Logger.log(TAG, "onLocalStatsUpdated updated : ", stats);
  }
  onRemoteStatsUpdated(id, stats) {
    //Logger.log(TAG, "onRemoteStatsUpdated updated : ", id);
  }

  onConnectionFailed() {
    //Logger.log(TAG,"onConnectionFailed Called......");
  }

  onDisconnect() {
    Logger.log(TAG, "onDisconnect Called......");
    connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,this.onConnectionSuccess
    );
    connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_FAILED,this.onConnectionFailed
    );
    connection.removeEventListener(
      JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,this.onDisconnect
    );
  }

  onDominantSpeakerChanged = (id) => {
    Logger.log(TAG, "onDominantSpeakerChanged id ...  ", id);
    Logger.log(
      TAG,
      "Participants who pinned me...  ",
      this.participantsWhoPinnedMe
    );
    if (!id) {
      return;
    }

    if (this.state.users[0].id !== id) {
      // alert("on dominant speaker changed");
      this.setState({
        dominantSpeakerId: id,
      });
      if (!this.state.pinnedUserId) {
        room.pinParticipant(id);
      }
    }
  };

  onUserJoined = (id, remoteUser) => {
    //Logger.log(TAG,"onUserJoined",remoteUser);
    const user = {
      id: remoteUser.getId(),
      name: remoteUser.getDisplayName(),
      status: remoteUser._connectionStatus,
      isAudioMuted: undefined,
      isVideoMuted: undefined,
      videoTrack: undefined,
      audioTrack: undefined,
    };
    this.setState({
      users: [...this.state.users, user],
    });
  };

  onUserLeft = (id, user) => {
    // console.log(TAG, "onUserLeft .....", user);
    let usersData = this.state.users.filter((u) => u.id !== user._id);
    // console.log(TAG, "onUserLeft after .....", usersData);

    this.setState({
      users: usersData,
    });
  };

  onTrackMuted = (track) => {
    //Logger.log(TAG,"onTrackMuted ...... ",track);
    let id = track.ownerEndpointId;
    if (track.isLocal()) {
      id = this.state.users[0].id;
    }

    const users = this.state.users.map((user) => {
      if (user.id === id) {
        if (track.getType() === "video") {
          return {
            ...user,
            videoTrack: track,
            isVideoMuted: track.isMuted(),
          };
        }
        return {
          ...user,
          audioTrack: track,
          isAudioMuted: track.isMuted(),
        };
      }

      return user;
    });

    this.setState({
      users,
    });
  };

  onRemoteTrack = (track) => {
    Logger.log(TAG, "onRemoteTrack ...... ", track);
    if (track.isLocal()) {
      return;
    }

    const userId = track.ownerEndpointId;

    if (isScreenSharingTrack(track)) {
      this.setState({pinnedUserId: userId});
    } else if (room.lastDominantSpeaker === userId) {
      this.setState({dominantSpeakerId: userId});
    }


    const users = this.state.users.map((user) => {
      if (user.id === userId) {
        if (track.type === "video") {
          return {
            ...user,
            videoTrack: track,
            isVideoMuted: track.muted,
          };
        } else {
          return {
            ...user,
            audioTrack: track,
            isAudioMuted: track.muted,
          };
        }
      }

      return user;
    });

    this.setState({
      users,
    });
  };

  onRemoteTrackRemoved = (track) => {
    Logger.log(TAG, "onRemoteTrackRemoved ...... ", track);
    const users = this.state.users.map((user) => {
      if (user.id === track.ownerEndpointId) {
        if (track.type === "video") {
          return {
            ...user,
            videoTrack: undefined,
          };
        } else {
          return {
            ...user,
            audioTrack: undefined,
          };
        }
      }

      return user;
    });

    this.setState({ users });
  };

  onConferenceJoined = async () => {
    //Logger.log("onConferenceJoined called ",this.state.localTracks);
    isJoined = true;

    const { localUser } = this;

    if (localUser.audioTrack) {
      room.addTrack(localUser.audioTrack);
    }

    if (localUser.videoTrack) {
      await room.addTrack(localUser.videoTrack);
    }
  };

  createLocalTracks = () => {
    JitsiMeetJS.createLocalTracks({
      devices: ["audio", "video"],
      facingMode: "user",
      resolution: 180,
      constraints: {
        video: {
          height: {
            ideal: 180,
            max: 180,
            min: 180,
          },
          maxFps: 15,
        },
      },
    })
      .then(this.onLocalTracks)
      .catch((error) => {
        JitsiMeetJS.createLocalTracks({
          devices: ["audio"],
          facingMode: "user",
        }).then(this.onLocalTracks);
      });
  };

  onLocalTracks = (tracks, isResumedFromScreenSharing) => {
    for (let i = 0; i < tracks.length; i++) {
      if (tracks[i].getType() === "video") {
        const users = [...this.state.users];
        const localUser = users[0];
        if (localUser.isVideoMuted && isResumedFromScreenSharing) {
          tracks[i].mute();
        }
        users[0] = {
          ...localUser,
          videoTrack: tracks[i],
        };
        this.setState({
          users,
        });

        // To handle Chrome/extension's stop button click
        if (isScreenSharingTrack(tracks[i])) {
          tracks[i].track.onended = () => {
            this.stopScreenShare();
          };
        }
      }
      if (tracks[i].getType() === "audio") {
        const users = [...this.state.users];
        const localUser = users[0];
        if (localUser.isAudioMuted && isResumedFromScreenSharing) {
          tracks[i].mute();
        }
        users[0] = {
          ...localUser,
          audioTrack: tracks[i],
        };
        this.setState({
          users,
        });
      }
      if (isJoined) {
        room.addTrack(tracks[i]);
      }
    }
  };

  toggleVideoButton = () => {
    const { localUser } = this;

    if(localUser.videoTrack){
      if (localUser.isVideoMuted) {
        localUser.videoTrack.unmute();
      } else {
        localUser.videoTrack.mute();
      }
    }
  };

  toggleAudioButton = () => {
    const { localUser } = this;
    if(localUser.audioTrack){
      if (localUser.isAudioMuted) {
        localUser.audioTrack.unmute();
      } else {
        localUser.audioTrack.mute();
      }
    }
  };

  onEndCallButtonCLicked = () => {
    this.unload();
    window.postMessage(
      window.parent.postMessage(
        JSON.stringify({
          type: "hangup",
          message: "Message text from iframe.",
        }),
        "*"
      ),
      "*"
    );
  };

  disposeLocalVideoTrack = async () => {
    const { localUser } = this;
    if (localUser.videoTrack) {
      await localUser.videoTrack.dispose();
    }
  };

  createScreenShareTrack = async () => {
    await this.disposeLocalVideoTrack();
    JitsiMeetJS.createLocalTracks({
      devices: ["audio", "desktop"],
      resolution: 360,
      constraints: {
        video: {
          height: {
            ideal: 360,
            max: 720,
            min: 180,
          },
          maxFps: 15,
        },
      },
    })
      .then((tracks) => {
        this.onLocalTracks(tracks);
        this.setState({
          isScreenShareOn: true,
        });
        room.sendCommand(SCREEN_SHARING_STARTED, {
          value: this.localUser.id,
        });
      })
      .catch((error) => {
        this.stopScreenShare();
        //Logger.log("Create Local error..");
      });
  };

  stopScreenShare = async () => {
    await this.disposeLocalVideoTrack();
    JitsiMeetJS.createLocalTracks({
      devices: ["audio", "video"],
      facingMode: "user",
      resolution: 360,
      constraints: {
        video: {
          height: {
            ideal: 360,
            max: 720,
            min: 180,
          },
          maxFps: 15,
        },
      },
    })
      .then((tracks) => {
        this.onLocalTracks(tracks, true);
        this.setState({
          isScreenShareOn: false,
        });
        room.sendCommand(SCREEN_SHARING_STOPPED, {
          value: this.localUser.id,
        });
      })
      .catch((error) => {
        // Logger.log("Create Local error..");
        throw error;
      });
  };

  onScreenShareClicked = () => {
    if (this.state.isScreenShareOn) {
      this.stopScreenShare();
    } else {
      this.createScreenShareTrack();
    }
  };

  onDisplayModeChange = () => {
    this.setState((state) => ({
      isGridMode: !state.isGridMode,
    }));
  };

  onUserItemClicked = (user) => {
    this.setState({
      dominantSpeakerId: user.id,
    });
  };


  renderLocalVideo = () =>{
    const { users} = this.state;
    const localUser = users[0];
    Logger.log(TAG,"renderLocalVideo for .... ",localUser);
    if(localUser.videoTrack){
        Logger.log(TAG,"local stream = ",localUser)
        return (
            <RTCView
              mirror = {this.state.isVideoMirrored}
              key={'localmedia'}
              objectFit = "cover"
              streamURL = { localUser.videoTrack.getOriginalStream().toURL() }
              style={{ height: 100, width: 100}}
              zOrder={2}
          /> 
        );
    }
  }

  renderMainVideo = () => {
    const { users, dominantSpeakerId, pinnedUserId } = this.state;
    const localUser = users[0];
    const dominantUser = users.find((user) => user.id === dominantSpeakerId);
    const pinnedUser = users.find((user) => user.id === pinnedUserId);
    this.mainVideoSpeaker = dominantUser || localUser;
    
    if (this.mainVideoSpeaker &&  this.mainVideoSpeaker.videoTrack) {
      let dominantVideoTrack = this.mainVideoSpeaker.videoTrack;

      if (pinnedUser && pinnedUser.videoTrack) {
        dominantVideoTrack = pinnedUser.videoTrack;
        this.mainVideoSpeaker = pinnedUser;
      }

      const isTrackOfTypeScreenSharing = isScreenSharingTrack(
        dominantVideoTrack
      );

    //   if (
    //     isTrackOfTypeScreenSharing &&
    //     dominantVideoTrack === localUser.videoTrack
    //   ) {
    //     return (
    //       <h1 style={{ color: "white" }}>You are presenting your screen</h1>
    //     );
    //   }
      console.log("Dominant video track status ",this.mainVideoSpeaker.status);
      
      if (this.mainVideoSpeaker && (this.mainVideoSpeaker.isVideoMuted || this.mainVideoSpeaker.status !== 'active' )) {
        return (
          <View style = {{height : '100%', width : '100%' , alignItems: 'center', justifyContent: 'center',flexDirection : 'column'}} >
            <TextAvatar
                backgroundColor={'#4d4c4f'}
                textColor={'#FFF'}
                size={100}
                type={'circle'} // optional
                >{this.mainVideoSpeaker.name}</TextAvatar>
            <Text style = {{fontSize : 18 , color : '#fff',padding: 20 }} >{this.mainVideoSpeaker.name}</Text>
          </View>
        );
      }

    Logger.log(TAG,"renderMainVideo with dominant video track.... ",dominantVideoTrack)
      if(dominantVideoTrack){
        if(dominantVideoTrack.isLocal()){
          return (
            <RTCView
                mirror = {this.state.isVideoMirrored}
                key={dominantSpeakerId}
                objectFit = { 'cover' }
                streamURL = { dominantVideoTrack.getOriginalStream().toURL() }
                style={{ height: "100%", width: "100%"}}
                zOrder = { 1 } />
          );
        }else{
          return (
            <RTCView
                key={dominantSpeakerId}
                objectFit = { 'cover' }
                streamURL = { dominantVideoTrack.getOriginalStream().toURL() }
                style={{ height: "100%", width: "100%"}}
                zOrder = { 1 } />
          );
        }
      }
    }
  };


  pinUser = (user) => {
    this.setState({
      pinnedUserId: user.id,
    });
    room.pinParticipant(user.id);
  };

  unPinUser = () => {
    room.pinParticipant(null);

    this.setState({
      pinnedUserId: null,
    });
  };

  onDoubleTap(){
    const { users} = this.state;
    const localUser = users[0];

    if(localUser.videoTrack){
        localUser.videoTrack._switchCamera();
        this.setState({
          isVideoMirrored : !this.state.isVideoMirrored
        })
    }
  }

  onPeopleClicked = () => {
      Logger.log(TAG,"onPeopleClicked");
      this.setState({
          showUserPopUP : !this.state.showUserPopUP
      })
  }

  renderPeopleButton(){
    if(!this.state.hideVideoSwitchButton){
    return <TouchableOpacity onPress={this.onPeopleClicked}>
      <View style={this.style.lt_button}>
        <Image
          style={this.style.lt_control_icon}
          source={require("./icons/icon_people.png")}
        />
      </View>
    </TouchableOpacity>
    }
  }

  renderUserItem(userItem){
    Logger.log(TAG,"Render User item : ",userItem);
    if(userItem){
        var user = userItem.item;
        var index = userItem.index;
        var username = user.name;
        var isPinned;

        if(index == 0){
            username += " (you)";
        }

        if (user.id === this.state.pinnedUserId) {
            isPinned = true;
        }else{
            isPinned = false;
        }

        return(
            <TouchableOpacity
                onPress={() => {
                    if (user.id === this.state.pinnedUserId) {
                        this.unPinUser();
                    } else {
                        this.pinUser(user);
                    }

                    this.setState({
                        showUserPopUP: false
                    });
                }}
                rippleColor="rgba(0, 0, 0, .20)">
                <View style={this.itemStyle(isPinned)}>
                        {/* <View style={this.style.lt_button}><Image style={this.style.lt_control_icon} source={require('./icons/icon_person.png')}/></View> */}
                        <TextAvatar
                            backgroundColor={'#4d4c4f'}
                            textColor={'#FFF'}
                            size={50}
                            type={'circle'} // optional
                            >{username}</TextAvatar>
                        <Text style={{fontSize : 18 , color : '#000' , marginStart : 10}}>{username}</Text>
                        <View style = {{flexDirection: 'row',justifyContent: 'flex-start',alignItems: 'center', position:"absolute" , right : 10}}>
                            {user.isAudioMuted ? <View style={this.style.lt_small_button}><Image style={{ width: 15, height: 15 }} source={require('./icons/icon_audio_mute.png')}/></View> : null}
                            {user.isVideoMuted ? <View style={this.style.lt_small_button}><Image style={{ width: 15, height: 15 }} source={require('./icons/icon_video_mute.png')}/></View> : null}
                        </View>
                </View>
            </TouchableOpacity>
                    
        )
    }
  }

//   getRandomColor = () =>
//     {
//         return 'rgb(' + (Math.floor(Math.random() * 256)) + ',' + (Math.floor(Math.random() * 256)) + ',' + (Math.floor(Math.random() * 256)) + ')';
//     }

  itemStyle = function(isPinned) {

    if(isPinned){
        return{
            padding: 15,
            fontSize: 18,
            height: 70,
            flexDirection: 'row',
            justifyContent: 'flex-start',
            alignItems: 'center',
            opacity:0.3,
            backgroundColor: '#4d4c4f'
        }
    }else{
        return {
            padding: 15,
            fontSize: 18,
            height: 70,
            flexDirection: 'row',
            justifyContent: 'flex-start',
            alignItems: 'center'
          }
    }
  }

  onAudioMute(){
      Logger.log(TAG,"on Audio Muted clicked");
      this.toggleAudioButton();
  }

  onVideoMute(){
    Logger.log(TAG,"on video Muted clicked");
    this.toggleVideoButton();
  }

  onHangUp(){
    Logger.log(TAG,"onHangUp clicked");
    this.unload();
    eventEmitter.emit('CallEvent', { type: 'EndCall' });
  }

  render() {
    const { users, pinnedUserId } = this.state;
    const localUser = users[0];
    
    return (
      <View>
        <DoubleTap onDoubleTap={this.onDoubleTap}>
            <View style={{ height: "100%" ,backgroundColor:'#282c34'}} >
              {this.renderMainVideo()}
            </View>
        </DoubleTap>
        

        <View style={{ height: 100, width: 100 , backgroundColor : '#000' ,position : 'absolute' , top : 0 , right : 0}} >
            {this.state.users[0] ? this.renderLocalVideo():''}
        </View>

        {this.state.showUserPopUP ? <View style = {this.style.container_people}>
                                        <Text style= {{fontSize : 20 , fontWeight : 'bold' , color : '#000' , padding: 10}}>  People({this.state.users.length}) </Text>

                                        <FlatList
                                            style= {{width : '100%'}}
                                            data={this.state.users}
                                            keyExtractor={user => user.id}
                                            renderItem={this.renderUserItem}/>
                                    </View>: null}

        <View style ={this.style.container}>
                <View style={this.style.container_background}
                />
                <View id="lt_control_container" style={ this.style.lt_control_container }>
                    { this.state.hideAudioMuteButton ? null : <AudioButton audioMute={localUser.isAudioMuted}  onAudioMute = {this.onAudioMute}/> }
                    { this.state.hidePauseVideoButton ? null : <VideoButton videoMute={localUser.isVideoMuted} onVideoMute = {this.onVideoMute}/> }
                    { this.state.hideEndCallButton ? null : <HangUpButton onHangUp={this.onHangUp}/> }
                    { this.renderPeopleButton()}
                    { this.state.hideAudioModeButton ? null : <AudioModeButton /> }
                </View>
            </View>
      </View>
    );
  }

  style = {
    lt_control_container: { 
        display:'flex', 
        width: '100%',
        flexDirection: 'row', 
        justifyContent: 'center', 
        alignItems: 'center', 
        flexWrap: 'wrap', 
        position: 'absolute',
        bottom: 7,
        zIndex: 5 
    },
    container_background:{
        display:'flex', 
        width: '100%',
        height: 90, 
        flexDirection: 'row', 
        justifyContent: 'center', 
        alignItems: 'center', 
        flexWrap: 'wrap', 
        position: 'absolute', 
        bottom: 0,
        zIndex: 2,
        opacity:0.5,
        borderTopLeftRadius: 20,
        borderTopRightRadius: 20,
        backgroundColor: '#2e2f30'
    },
    container_people:{
        display:'flex', 
        width: '80%',
        height: '70%',
        marginTop: 110,
        marginRight : 10,
        flexDirection: 'column', 
        justifyContent: 'center', 
        alignItems: 'center', 
        flexWrap: 'wrap', 
        position: 'absolute',
        zIndex: 2,
        right : 0,
        borderRadius :10,
        backgroundColor: '#FFF'
    },
    item: {
        padding: 15,
        fontSize: 18,
        height: 70,
        flexDirection: 'row',
        justifyContent: 'flex-start',
        alignItems: 'center'
    },
    item_pinned : {
        padding: 15,
        fontSize: 18,
        height: 70,
        flexDirection: 'row',
        justifyContent: 'flex-start',
        alignItems: 'center',
        opacity:0.5,
        backgroundColor: '#f66301'
    },
    container: Platform.OS === 'ios' ? { zIndex: 2 }  : { },
    lt_control_icon: { width: 25, height: 25 },
    lt_button: { width: 55, height: 55, margin: 5, backgroundColor: '#4d4c4f', borderRadius: 50, alignItems: 'center', justifyContent:'center' },
    lt_small_button: { width: 20, height: 20, margin: 5, backgroundColor: '#ec4c3a', borderRadius: 20, alignItems: 'center', justifyContent:'center' },
};
}