import React, { Component } from "react";
import {
  View,
  TouchableOpacity,
  Image,
  NativeEventEmitter,
} from "react-native";
import AudioMode from "../AudioMode";
const AudioModeEmitter = new NativeEventEmitter(AudioMode);

const devices = {
  HEADPHONES: "HEADPHONES",
  SPEAKER: "SPEAKER",
  EARPIECE: "EARPIECE",
  BLUETOOTH: "BLUETOOTH",
};
class AudioModeButton extends Component {
  state = {
    selectedDevice: null,
    availableDevices: {},
  };

  constructor(props) {
    super(props);
    AudioMode.setMode(AudioMode.VIDEO_CALL);
  }

  componentDidMount() {
    AudioModeEmitter.addListener(AudioMode.DEVICE_CHANGE_EVENT, (devices) => {
      let availableDevices = {};
      let selectedDevice;
      devices.forEach((device) => {
        availableDevices[device.type] = device;
        if (device.selected) {
          selectedDevice = device.type;
        }
      });

      if (selectedDevice) {
        this.setState({
          selectedDevice,
          availableDevices,
        });
      }
    });
  }

  getDeviceByPriority = () => {
    if (this.state.availableDevices[devices.HEADPHONES]) {
      return devices.HEADPHONES;
    } else if (this.state.availableDevices[devices.BLUETOOTH]) {
      return devices.BLUETOOTH;
    } else if (this.state.availableDevices[devices.EARPIECE]) {
      return devices.EARPIECE;
    } else {
      return this.state.availableDevices[devices.SPEAKER];
    }
  };

  routeAudioToSpeaker = () => {
    AudioMode.setAudioDevice(devices.SPEAKER);
  };

  routeAudioToDefaultMode = () => {
    AudioMode.setAudioDevice(this.getDeviceByPriority());
  };

  renderSpeakerIcon() {
    return (
      <TouchableOpacity onPress={this.routeAudioToDefaultMode}>
        <Image
          style={this.style.lt_control_icon}
          source={require("../icons/speaker_phone.png")}
        />
      </TouchableOpacity>
    );
  }

  renderEarPieceIcon() {
    return (
      <TouchableOpacity onPress={this.routeAudioToSpeaker}>
        <Image
          style={this.style.lt_control_icon}
          source={require("../icons/icon_earpiece.png")}
        />
      </TouchableOpacity>
    );
  }

  renderIcon = () => {
    switch (this.state.selectedDevice) {
      case devices.SPEAKER:
        return this.renderSpeakerIcon();
      case devices.BLUETOOTH:
      case devices.EARPIECE:
      case devices.HEADPHONES:
        return this.renderEarPieceIcon();
    }
  };

  render() {
    return <View style={this.style.lt_control_view}>{this.renderIcon()}</View>;
  }

  style = {
    lt_control_icon: { width: 25, height: 25 },
    lt_control_view: {
      width: 55,
      height: 55,
      margin: 10,
      backgroundColor: "#4d4c4f",
      borderRadius: 50,
      alignItems: "center",
      justifyContent: "center",
    },
  };
}

export default AudioModeButton;
